{% load i18n %}
<!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.01 Transitional//EN" "http://www.w3.org/TR/html4/loose.dtd">
<html>
    <head>
        <title>{% trans 'Maintenance mode' %}</title>
    </head>
    <body>
        <p>{% trans "System is in the maintenance mode right now. Please, come back later." %}</p>
    </body>
</html>