{% load i18n %}

{% if messages %}
<div class="messages">
    {% for message in messages %}
    <div class="alert alert-{{ message.tags }}">
        <div class="wrapper">
            {% autoescape off %}{{ message }}{% endautoescape %}
        </div><!-- /.wrapper -->
    </div>
    {% endfor %}
</div><!-- /#messages -->
{% endif %}