# Generated by Django 2.2.9 on 2020-12-07 11:14

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('pragmatic', '0001_initial'),
    ]

    operations = [
        migrations.AlterField(
            model_name='deletedobject',
            name='object_str',
            field=models.CharField(max_length=300),
        ),
    ]
